<?php

/**
 * @file
 * Theme for Gantt views.
 */

use Drupal\Core\Link;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;

/**
 * Template preprocess views gantt.
 *
 * @param array $variables
 *   Array variable.
 */
function template_preprocess_views_view_gantt(array &$variables) {
  $view = $variables['view'];
  $style = $view->style_plugin;
  $setting = $style->options;
  $fieldDate = $view->getHandlers('field')[$setting["start_date"]];
  $dateType = \Drupal::entityTypeManager()
    ->getStorage('date_format')
    ->load($fieldDate["settings"]["format_type"]);
  $patterns = !empty($dateType) ? explode(' ', $dateType->getPattern()) : ['d-m-Y'];
  $patterns = array_filter(
    $patterns,
    function ($value) {
      return strlen($value) >= 4;
    }
  );
  $setting['format_date'] = current($patterns);
  $variables["view_id"] = $view->storage->id();
  $variables["display_id"] = $view->current_display;
  if ($setting["cdn"]) {
    $variables['view']->element['#attached']['library'][] = 'gantt/gantt.cdn';
  }
  else {
    $variables['view']->element['#attached']['library'][] = 'gantt/gantt';
  }
  $id = implode('-', ["gantt", $view->id(), $view->current_display]);
  $variables["id_gantt"] = $id;
  $currentPath = \Drupal::service('path.current')->getPath();
  $variables['view']->element['#attached']['drupalSettings']['gantt'][$id] = [
    'date_format' => str_replace(
      ['Y', 'm', 'd', 'H', 'i', 's'],
      ['%Y', '%m', '%d', '%H', '%i', '%s'],
      $setting['format_date']
    ),
    'id' => $variables["id_gantt"],
    'owner' => $setting['owner'],
    'ajax' => Url::fromRoute('gantt.ajax', [
      'view_id' => $variables["view_id"],
      'display_id' => $variables["display_id"],
    ])->toString(),
    'import' => Url::fromRoute('gantt.import', [
      'view_id' => $variables["view_id"],
      'display_id' => $variables["display_id"],
    ])->toString(),
    'current_path' => 'destination=' . $currentPath,
    'add_task' => $setting["add_task"],
    'edit_task' => $setting["edit_task"],
    'native_dialog' => $setting["native_dialog"],
    'add_link' => '',
    'edit_link' => '',
  ];
  $variables["add_link"] = '';
  if (!empty($variables['rows'])) {
    $entityFirst = current($variables['rows'])->_entity;
    $setting['entity_type'] = $entityFirst->getEntityTypeId();
    if ($setting["native_dialog"]) {
      $addLinkText = Markup::create('<i class="bi bi-plus-lg"></i> ' . t('Add'));
      $destination = [
        'query' => ['destination' => $currentPath],
      ];
      if ($setting['entity_type'] == 'node') {
        $uriAdd = Url::fromRoute('node.add', [
          'node_type' => $setting['entity_type'],
        ], $destination);
        $uriEdit = '/node';
      }
      else {
        $uriAdd = Url::fromRoute('gantt.add', [
          'paragraph_type' => $entityFirst->bundle(),
          'entity_type' => $entityFirst->get('parent_type')->value,
          'entity_field' => $entityFirst->get('parent_field_name')->value,
          'entity_id' => $entityFirst->get('parent_id')->value,
        ], $destination);
        $uriEdit = '/paragraphs_item';
      }
      if ($setting["add_task"]) {
        $variables["add_link"] = Link::fromTextAndUrl($addLinkText, $uriAdd)
          ->toString();
        $variables['view']->element['#attached']['drupalSettings']['gantt'][$id]["add_link"] = $uriAdd->toString();
      }
      if ($setting["add_task"]) {
        $variables['view']->element['#attached']['drupalSettings']['gantt'][$id]["edit_link"] = $uriEdit;
      }
    }
  }
  $data = convert_gantt_data($variables['rows'], $setting);
  $variables["data"] = ['data' => $data['data']['data']];
  $variables["dataActual"] = !empty($data['data']['actual']) ? ['data' => $data['data']['actual']] : [];
  if (!empty($data['links'])) {
    $variables["data"]['links'] = $data['links'];
    if (!empty($variables["dataActual"])) {
      $variables["dataActual"]['links'] = $data['links'];
    }
  }
  $variables["owner"] = [];
  // Set session variable for import paragraphs.
  if (!empty($view->argument)) {
    foreach ($view->argument as $key => $arg) {
      if ($key == 'parent_id') {
        $variables["paragraph_parent_id"] = $view->args[$arg->position];
        if (!empty($entityFirst)) {
          $variables["paragraph_parent_type"] = current($entityFirst->get('parent_type'))[0]->value;
          $variables["paragraph_parent_field_name"] = current($entityFirst->get('parent_field_name'))[0]->value;
        }
      }
    }
  }
}

/**
 * {@inheritDoc}
 */
function convert_gantt_data($items, array $setting) {
  $links = [];
  $outData = [];
  $mappingGantt = [
    'text',
    'type',
    'start_date',
    'duration',
    'progress',
    'open',
    'parent',
    'owner',
  ];
  $fieldDateType = 'date';
  if (!empty($setting['group_field'])) {
    $bundle_info = \Drupal::service('entity_type.bundle.info');
    $entity_types = $bundle_info->getBundleInfo($setting['entity_type']);
  }
  foreach ($items as $delta => $item) {
    $entity = $item->_entity;
    $group = 0;
    if (!empty($setting['group_field'])) {
      $group = $entity->get($setting["group_field"])->getString();
    }
    $fieldDefinitions = $entity->getFieldDefinitions();
    if (!empty($fieldDefinitions[$setting['start_date']])) {
      $fieldDateType = $fieldDefinitions[$setting['start_date']]->getType();
    }
    $data[$group][$delta] = ['id' => $entity->id()];
    foreach ($mappingGantt as $ganttField) {
      if (!empty($field_name = $setting[$ganttField])) {
        $value = $entity->get($field_name)->getString();
        if (!empty($value)) {
          $data[$group][$delta][$ganttField] = $value;
        }
        if ($ganttField == 'start_date') {
          if (!empty($entity->get($field_name)->date)) {
            $data[$group][$delta][$ganttField] = $entity->get($field_name)->date->format($setting['format_date']);
          }
          if ($fieldDateType == 'daterange' && $dateValue = $entity->get($field_name)->value) {
            $data[$group][$delta][$ganttField] = date('d-m-Y H:i', strtotime($dateValue));
          }
        }

        // Process owner.
        if ($ganttField == 'owner' && !$entity->get($field_name)->isEmpty()) {
          $values = $entity->get($field_name)->getValue();
          $data[$group][$delta][$ganttField] = [];
          foreach ($values as $value) {
            if (!empty($value->entity)) {
              $data[$group][$delta][$ganttField][] = $value->entity->id();
            }
          }
          // @todo remove later for multiuser.
          if (!empty($value->entity)) {
            $data[$group][$delta][$ganttField] = $value->entity->id();
          }
        }
      }
    }

    // Actual date.
    if (!empty($field_name = $setting['actual']['actual_date']) && !empty($fieldDefinitions[$setting['actual']['actual_date']])) {
      $actual[$group][$delta] = $data[$group][$delta];
      $actual[$group][$delta]['start_date'] = NULL;
      $actual[$group][$delta]['duration'] = 1;

      if (!empty($entity->get($field_name)->date)) {
        $data[$group][$delta]['start_date'] = $entity->get($field_name)->date->format('d-m-Y H:i');
      }
      $fieldActualDateType = $fieldDefinitions[$field_name]->getType();
      if ($fieldActualDateType == 'daterange' && $dateValue = $entity->get($field_name)->value) {
        $actual[$group][$delta]['start_date'] = date('d-m-Y H:i', strtotime($dateValue));
      }
      if (!empty($field_name = $setting['actual']['actual_duration'])) {
        $value = $entity->get($field_name)->getString();
        if (!empty($value)) {
          $actual[$group][$delta]['duration'] = $value;
        }
      }
      if (empty($actual[$group][$delta]['start_date'])) {
        unset($actual[$group][$delta]);
      }
    }

    // Process Links.
    if (!empty($setting['links'])) {
      $valueLinks = $entity->get($setting['links'])->getValue();
      foreach ($valueLinks as $valueLink) {
        $links[] = [
          'id' => implode('-', [$entity->id(), $valueLink['first']]),
          'source' => $entity->id(),
          'target' => (int) $valueLink['first'],
          'type' => (int) $valueLink['second'],
        ];
      }
    }
  }
  if (count($data) == 1) {
    $data = current($data);
    if (!empty($actual)) {
      $actual = current($actual);
    }
  }
  else {
    $originData = $data;
    $data = [];
    foreach ($originData as $group => $tasks) {
      $groupName = $group;
      if (!empty($entity_types[$group])) {
        $groupName = $entity_types[$group]['label'];
      }
      $data[] = [
        'id' => $group,
        'text' => $groupName,
        'type' => 'project',
        'open' => TRUE,
        'parent' => 0,
      ];
      foreach ($tasks as $delta => $task) {
        if (empty($task['parent'])) {
          $task['parent'] = $group;
        }
        $data[] = $task;
      }
    }

    if (!empty($actual)) {
      $originActual = $actual;
      $actual = [];
      foreach ($originActual as $group => $tasks) {
        $groupName = $group;
        if (!empty($entity_types[$group])) {
          $groupName = $entity_types[$group]['label'];
        }
        $actual[] = [
          'id' => $group,
          'text' => $groupName,
          'type' => 'project',
          'open' => TRUE,
          'parent' => 0,
        ];
        foreach ($tasks as $delta => $task) {
          if (empty($task['parent'])) {
            $task['parent'] = $group;
          }
          $actual[] = $task;
        }
      }
    }
  }
  if (!empty($data)) {
    $outData['data'] = ['data' => array_values($data)];
    if (!empty($actual)) {
      $outData['data']['actual'] = array_values($actual);
    }
    if (!empty($links)) {
      $outData['links'] = $links;
    }
  }
  return $outData;
}
